@@ -6,6 +6,11 @@
 
 #include "VirtualDesktopSwitchPlugin.h"
 
+#include <QDBusConnection>
+#include <QDBusMessage>
+#include <QDBusPendingCall>
+#include <QDBusPendingCallWatcher>
+#include <QDBusReply>
 #include <QString>
 
 #include <kwindowsystem.h>
@@ -13,6 +18,8 @@
 KAMD_EXPORT_PLUGIN(virtualdesktopswitchplugin, VirtualDesktopSwitchPlugin, "kactivitymanagerd-plugin-virtualdesktopswitch.json")
 
 const auto configPattern = QStringLiteral("desktop-for-%1");
+const auto kwinName = QStringLiteral("org.kde.KWin");
+const auto kwinPath = QStringLiteral("/KWin");
 
 VirtualDesktopSwitchPlugin::VirtualDesktopSwitchPlugin(QObject *parent, const QVariantList &args)
     : Plugin(parent)
@@ -47,15 +54,29 @@ void VirtualDesktopSwitchPlugin::currentActivityChanged(const QString &activity)
         return;
     }
 
-    config().writeEntry(configPattern.arg(m_currentActivity), QString::number(KWindowSystem::currentDesktop()));
+    const auto desktopId = config().readEntry(configPattern.arg(activity), -1);
 
-    m_currentActivity = activity;
-
-    const auto desktopId = config().readEntry(configPattern.arg(m_currentActivity), -1);
-
-    if (desktopId <= KWindowSystem::numberOfDesktops() && desktopId >= 0) {
-        KWindowSystem::setCurrentDesktop(desktopId);
+    if (KWindowSystem::isPlatformX11()) {
+        config().writeEntry(configPattern.arg(m_currentActivity), QString::number(KWindowSystem::currentDesktop()));
+        if (desktopId <= KWindowSystem::numberOfDesktops() && desktopId >= 0) {
+            KWindowSystem::setCurrentDesktop(desktopId);
+        }
+    } else {
+        auto currentMessage = QDBusMessage::createMethodCall(kwinName, kwinPath, kwinName, QStringLiteral("currentDesktop"));
+        auto currentDesktopCall = QDBusConnection::sessionBus().asyncCall(currentMessage);
+        if (desktopId >= 0) {
+            auto setDesktopMessage = QDBusMessage::createMethodCall(kwinName, kwinPath, kwinName, QStringLiteral("setCurrentDesktop"));
+            setDesktopMessage.setArguments({desktopId});
+            QDBusConnection::sessionBus().send(setDesktopMessage);
+        }
+        const auto watcher = new QDBusPendingCallWatcher(currentDesktopCall);
+        connect(watcher, &QDBusPendingCallWatcher::finished, this, [oldActivity = m_currentActivity, watcher, this] {
+            watcher->deleteLater();
+            auto reply = QDBusReply<int>(*watcher);
+            config().writeEntry(configPattern.arg(oldActivity), reply.isValid() ? reply.value() : 0);
+        });
     }
+    m_currentActivity = activity;
 }
 
 void VirtualDesktopSwitchPlugin::activityRemoved(const QString &activity)
